==> lagoo-0.1.6.tar.gz: lagoo-0.1.6/src/lagoo/lines.py <==
# -*- coding:utf-8 -*-
"""
Author: fandi.di@bytedance.com
"""
import tensorflow as tf
import lagoo.feature_column as fc_lib


def _parse_line(line, feature_list, label_cols, feature_columns, bucket_map, line_default_type_list, selected_indexes,
                need_encode_map, pre_emb_len_map, field_delim='\t', na_value='nan'):
    """ Mapping Function
        Required Input
            - line            = Dataset
            - feature_list    = List
            - record_defaults = List
        Expected Output -
            - feature_dict
            - label_dict
    """
    fields = tf.io.decode_csv(line, record_defaults=line_default_type_list, select_cols=selected_indexes,
                              field_delim=field_delim, na_value=na_value)
    field_dict = dict(zip(feature_list, fields))
    feature_dict = {}

    for feat_col in feature_columns:
        if feat_col.name not in need_encode_map:
            if isinstance(feat_col, fc_lib.VarLenSparseFeat):
                feat_ids = tf.strings.split([field_dict[feat_col.name]], ',').values[:feat_col.maxlen]
                feat_ids = tf.reshape(feat_ids, shape=[-1])
                if feat_col.dtype != 'string':
                    feat_ids = tf.strings.to_number(feat_ids, out_type=tf.int32)
                feature_dict[feat_col.name] = feat_ids
            elif isinstance(feat_col, fc_lib.SparseFeat):
                feature_dict[feat_col.name] = field_dict[feat_col.name]
            elif isinstance(feat_col, fc_lib.DenseFeat):
                if feat_col.name in pre_emb_len_map:
                    v = tf.strings.split([field_dict[feat_col.name]], ',').values
                    v = tf.reshape(v, shape=[-1])
                    v = tf.strings.to_number(v, out_type=tf.float32)
                    feature_dict[feat_col.name] = v
                else:
                    feature_dict[feat_col.name] = field_dict[feat_col.name]
        else:  # need_encode
            if isinstance(feat_col, fc_lib.VarLenSparseFeat):  # if 'VarLenSparseFeat' in str(feat_col):
                feat_ids = tf.strings.to_hash_bucket_strong(tf.strings.split([field_dict[feat_col.name]], ','),
                                                            2 * len(need_encode_map[feat_col.name]), [1, 2]).values
                # feat_ids = tf.strings.split([field_dict[feat_col.name]], ',').values#[:feat_col.maxlen]

                # feat_ids = tf.argmax(tf.cast(feat_ids[1] == need_encode_map[feat_col.name], tf.int32))

                # for i in range(1, feat_col.maxlen):
                #     if feat_ids[i] == '':
                #         feat_ids = tf.concat([feat_ids, tf.argmax(tf.cast(feat_ids[i] == need_encode_map[feat_col.name], tf.int32))])
                # feat_ids = tf.reshape(feat_ids, shape=[-1])
                # feat_ids = tf.reshape(feat_ids, shape=[-1])
                # encode_feat_ids = tf.map_fn(fn=lambda x: tf.argmax(tf.cast(x == need_encode_map[feat_col.name], tf.int32)), elems=feat_ids)
                # encode_feat_ids = tf.reshape(encode_feat_ids, shape=[-1])
                # if feat_col.dtype != 'string':
                #     feat_ids = tf.strings.to_number(feat_ids, out_type=tf.int32)
                feature_dict[feat_col.name] = feat_ids
            elif isinstance(feat_col, fc_lib.SparseFeat):
                feature_dict[feat_col.name] = tf.strings.to_hash_bucket_strong(field_dict[feat_col.name],
                                                                               2 * len(need_encode_map[feat_col.name]),
                                                                               [1, 2])  # .values
                # feature_dict[feat_col.name] = tf.argmax(tf.cast(field_dict[feat_col.name] == need_encode_map[feat_col.name], tf.int32))

    # buckets
    for ft in list(bucket_map.keys()):
        feature_dict[ft] = tf.raw_ops.Bucketize(input=feature_dict[ft], boundaries=bucket_map[ft])

    labels = {label: field_dict[label] for label in label_cols}
    return feature_dict, labels


def gen_dataset(filepath, feature_list, label_cols, feature_columns, bucket_map, line_default_type_list,
                selected_indexes, need_encode_map, pre_emb_len_map, field_delim='\t', na_value='nan', shuffle=True,
                batch_size=5, shuffle_buffer_size=15000):
    """ Generate Dataset """
    # # padding
    # pad_shapes = {}
    # pad_values = {}
    # for fea_col in feature_columns:
    #     if 'VarLen' in str(fea_col):
    #         max_tokens = fea_col.maxlen
    #         pad_shapes[fea_col.name] = tf.TensorShape([max_tokens])
    #         pad_values[fea_col.name] = '-1' if fea_col.dtype == 'string' else -1
    #     # no need to pad labels
    #     elif 'VarLen' not in str(fea_col) and 'Sparse' in str(fea_col):
    #         pad_values[fea_col.name] = '-1' if fea_col.dtype == 'string' else -1
    #         pad_shapes[fea_col.name] = tf.TensorShape([])
    #     else:
    #         pad_values[fea_col.name] = 0.0
    #         pad_shapes[fea_col.name] = tf.TensorShape([fea_col.dim])
    # pad_shapes = (pad_shapes, (tf.TensorShape([]), tf.TensorShape([])))
    # pad_values = (pad_values, (tf.constant(0, dtype=tf.int32), tf.constant(0.0, dtype=tf.float32)))

    # list_files
    if '*' in filepath or isinstance(filepath, list):
        dataset_list = tf.data.Dataset.list_files(filepath)
        dataset = dataset_list.flat_map(lambda file: tf.data.TextLineDataset(file).skip(1))
    else:
        dataset = tf.data.TextLineDataset(filepath).skip(1)
    if shuffle:
        return dataset.shuffle(buffer_size=shuffle_buffer_size).map(
            lambda x: _parse_line(x, feature_list=feature_list, label_cols=label_cols,
                                  feature_columns=feature_columns, bucket_map=bucket_map,
                                  line_default_type_list=line_default_type_list, selected_indexes=selected_indexes,
                                  field_delim=field_delim, na_value=na_value, need_encode_map=need_encode_map,
                                  pre_emb_len_map=pre_emb_len_map),
            num_parallel_calls=tf.data.experimental.AUTOTUNE).padded_batch(batch_size=batch_size).prefetch(
            buffer_size=tf.data.experimental.AUTOTUNE)
    else:
        return dataset.map(
            lambda x: _parse_line(x, feature_list=feature_list, label_cols=label_cols,
                                  feature_columns=feature_columns, bucket_map=bucket_map,
                                  line_default_type_list=line_default_type_list,
                                  selected_indexes=selected_indexes,
                                  field_delim=field_delim, na_value=na_value, need_encode_map=need_encode_map,
                                  pre_emb_len_map=pre_emb_len_map),
            num_parallel_calls=tf.data.experimental.AUTOTUNE).padded_batch(batch_size=batch_size).prefetch(
            buffer_size=tf.data.experimental.AUTOTUNE)

==> lagoo-0.1.9.tar.gz: lagoo-0.1.9/src/lagoo/lines.py <==
# -*- coding:utf-8 -*-
"""
Author: fandi.di@bytedance.com
"""
import tensorflow as tf
import lagoo.feature_column as fc_lib


def _parse_line(line, feature_list, label_cols, feature_columns, bucket_map, line_default_type_list, selected_indexes,
                need_encode_map, pre_emb_len_map, field_diy_tf_kv_map, field_delim='\t', na_value='nan'):
    """ Mapping Function
        Required Input
            - line            = Dataset
            - feature_list    = List
            - record_defaults = List
        Expected Output -
            - feature_dict
            - label_dict
    """
    fields = tf.io.decode_csv(line, record_defaults=line_default_type_list, select_cols=selected_indexes,
                              field_delim=field_delim, na_value=na_value)
    field_dict = dict(zip(feature_list, fields))
    feature_dict = {}

    for feat_col in feature_columns:
        if feat_col.name not in need_encode_map:
            if isinstance(feat_col, fc_lib.VarLenSparseFeat):
                feat_ids = tf.strings.split([field_dict[feat_col.name]], ',').values[:feat_col.maxlen]
                feat_ids = tf.reshape(feat_ids, shape=[-1])
                if feat_col.dtype != 'string':
                    feat_ids = tf.strings.to_number(feat_ids, out_type=tf.int32)
                feature_dict[feat_col.name] = feat_ids
            elif isinstance(feat_col, fc_lib.SparseFeat):
                feature_dict[feat_col.name] = field_dict[feat_col.name]
            elif isinstance(feat_col, fc_lib.DenseFeat):
                if feat_col.name in pre_emb_len_map:
                    v = tf.strings.split([field_dict[feat_col.name]], ',').values
                    v = tf.reshape(v, shape=[-1])
                    v = tf.strings.to_number(v, out_type=tf.float32)
                    feature_dict[feat_col.name] = v
                else:
                    feature_dict[feat_col.name] = field_dict[feat_col.name]
        else:  # need_encode
            if feat_col.name in field_diy_tf_kv_map:  # diy field map
                feat_ids = field_diy_tf_kv_map[feat_col.name][field_dict[feat_col.name]]
                feat_ids = tf.reshape(feat_ids, shape=[-1])
                feature_dict[feat_col.name] = feat_ids
            else:
                if isinstance(feat_col, fc_lib.VarLenSparseFeat):  # if 'VarLenSparseFeat' in str(feat_col):
                    feat_ids = tf.strings.to_hash_bucket_strong(tf.strings.split([field_dict[feat_col.name]], ','),
                                                                2 * len(need_encode_map[feat_col.name]), [1, 2]).values
                    feature_dict[feat_col.name] = feat_ids
                elif isinstance(feat_col, fc_lib.SparseFeat):
                    feature_dict[feat_col.name] = tf.strings.to_hash_bucket_strong(field_dict[feat_col.name],
                                                                                   2 * len(
                                                                                       need_encode_map[feat_col.name]),
                                                                                   [1, 2])  # .values
    # buckets
    for ft in list(bucket_map.keys()):
        feature_dict[ft] = tf.raw_ops.Bucketize(input=feature_dict[ft], boundaries=bucket_map[ft])
    labels = {label: field_dict[label] for label in label_cols}
    return feature_dict, labels


def gen_dataset(filepath, feature_list, label_cols, feature_columns, bucket_map, line_default_type_list,
                selected_indexes, need_encode_map, pre_emb_len_map, field_diy_map,
                field_delim='\t', na_value='nan', shuffle=True,
                batch_size=5, shuffle_buffer_size=15000):
    """ Generate Dataset """
    # # padding
    # pad_shapes = {}
    # pad_values = {}
    # for fea_col in feature_columns:
    #     if 'VarLen' in str(fea_col):
    #         max_tokens = fea_col.maxlen
    #         pad_shapes[fea_col.name] = tf.TensorShape([max_tokens])
    #         pad_values[fea_col.name] = '-1' if fea_col.dtype == 'string' else -1
    #     # no need to pad labels
    #     elif 'VarLen' not in str(fea_col) and 'Sparse' in str(fea_col):
    #         pad_values[fea_col.name] = '-1' if fea_col.dtype == 'string' else -1
    #         pad_shapes[fea_col.name] = tf.TensorShape([])
    #     else:
    #         pad_values[fea_col.name] = 0.0
    #         pad_shapes[fea_col.name] = tf.TensorShape([fea_col.dim])
    # pad_shapes = (pad_shapes, (tf.TensorShape([]), tf.TensorShape([])))
    # pad_values = (pad_values, (tf.constant(0, dtype=tf.int32), tf.constant(0.0, dtype=tf.float32)))
    field_diy_tf_kv_map = {x:
        tf.lookup.StaticVocabularyTable(
            tf.lookup.KeyValueTensorInitializer(
                keys=tf.constant(field_diy_map[x]),
                values=tf.constant(list(range(len(field_diy_map[x]))), dtype=tf.int64)
            ),
            num_oov_buckets=1
        )
        for x in field_diy_map
    }
    # list_files
    if '*' in filepath or isinstance(filepath, list):
        dataset_list = tf.data.Dataset.list_files(filepath)
        dataset = dataset_list.flat_map(lambda file: tf.data.TextLineDataset(file).skip(1))
    else:
        dataset = tf.data.TextLineDataset(filepath).skip(1)
    if shuffle:
        return dataset.shuffle(buffer_size=shuffle_buffer_size).map(
            lambda x: _parse_line(x, feature_list=feature_list, label_cols=label_cols,
                                  feature_columns=feature_columns, bucket_map=bucket_map,
                                  line_default_type_list=line_default_type_list, selected_indexes=selected_indexes,
                                  field_delim=field_delim, na_value=na_value, need_encode_map=need_encode_map,
                                  pre_emb_len_map=pre_emb_len_map, field_diy_tf_kv_map=field_diy_tf_kv_map),
            num_parallel_calls=tf.data.experimental.AUTOTUNE).padded_batch(batch_size=batch_size).prefetch(
            buffer_size=tf.data.experimental.AUTOTUNE)
    else:
        return dataset.map(
            lambda x: _parse_line(x, feature_list=feature_list, label_cols=label_cols,
                                  feature_columns=feature_columns, bucket_map=bucket_map,
                                  line_default_type_list=line_default_type_list,
                                  selected_indexes=selected_indexes,
                                  field_delim=field_delim, na_value=na_value, need_encode_map=need_encode_map,
                                  pre_emb_len_map=pre_emb_len_map, field_diy_tf_kv_map=field_diy_tf_kv_map),
            num_parallel_calls=tf.data.experimental.AUTOTUNE).padded_batch(batch_size=batch_size).prefetch(
            buffer_size=tf.data.experimental.AUTOTUNE)
